# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/ml.peft.ipynb.

# %% auto 0
__all__ = ['log', 'merge_and_publish']

# %% ../../nbs/ml.peft.ipynb 3
import torch
from ..logging import get_logger

log = get_logger(__name__)

# %% ../../nbs/ml.peft.ipynb 4
def merge_and_publish(model_id: str, torch_dtype=torch.float16, device_map={"": 0}, merged_model_id=None):
    from peft import AutoPeftModelForCausalLM
    from transformers import AutoTokenizer
    log.info(f"Loading model and tokenizer for {model_id}") 
    model = AutoPeftModelForCausalLM.from_pretrained(
        model_id,
        torch_dtype=torch_dtype,
        device_map=device_map,
    )
    tokenizer = AutoTokenizer.from_pretrained(model_id, trust_remote_code=True)
    
    log.info("Merging adapters to model...") 
    model = model.merge_and_unload()

    if merged_model_id is None:
        merged_model_id = f"{model_id}-merged"
    log.info(f"Pushing merged model to HF hub as {merged_model_id}") 
    model.push_to_hub(merged_model_id)
    tokenizer.push_to_hub(merged_model_id)
    return merged_model_id



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/testing.ipynb.

# %% auto 0
__all__ = ['TEST_IMAGE', 'fixture_directory', 'fixture_path']

# %% ../nbs/testing.ipynb 3
from pathlib import Path

# %% ../nbs/testing.ipynb 4
def fixture_directory() -> Path:
    cwd = Path('.').resolve()
    if cwd.name == 'nbs':
        return cwd / 'fixtures'
    return cwd / 'nbs' / 'fixtures'


def fixture_path(relative_path) -> Path:
    return fixture_directory() / relative_path

# %% ../nbs/testing.ipynb 5
TEST_IMAGE = fixture_path('puppy.jpg')



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/ml.llama.ipynb.

# %% auto 0
__all__ = ['LLAMA2_CHAT_PROMPT_TEMPLATE', 'prepare_llama2_for_training', 'prepare_llama2_for_inference']

# %% ../../nbs/ml.llama.ipynb 3
LLAMA2_CHAT_PROMPT_TEMPLATE = """<s>[INST] <<SYS>>
{system_prompt}
<</SYS>>

{user_message} [/INST]"""

# %% ../../nbs/ml.llama.ipynb 4
def prepare_llama2_for_training(tokenizer, model):
    tokenizer.pad_token = tokenizer.eos_token
    tokenizer.padding_side = "right"  # Fix weird overflow issue with fp16 training
    model.config.pretraining_tp = 1
    model.config.use_cache = False

def prepare_llama2_for_inference(tokenizer, model):
    tokenizer.pad_token = tokenizer.eos_token
    tokenizer.padding_side = "right"  # Fix weird overflow issue with fp16 training
    model.config.use_cache = True



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/ml.clip.ipynb.

# %% auto 0
__all__ = ['load_clip_preprocess', 'make_tfms_from_clip_preprocess', 'ClipClassificationHead', 'ClipZeroShotClassifier']

# %% ../../nbs/ml.clip.ipynb 3
import clip
import torch
import torch.nn as nn
from torchvision import transforms
from .vision import TorchVisionTransform

# %% ../../nbs/ml.clip.ipynb 4
def load_clip_preprocess(clip_model_name):
    from clip import clip
    return clip.load(clip_model_name, device='cpu')[1]


# %% ../../nbs/ml.clip.ipynb 5
def make_tfms_from_clip_preprocess(clip_preprocess):
    item_tfms = TorchVisionTransform(transforms.Compose(clip_preprocess.transforms[:-2]))
    batch_tfms = TorchVisionTransform(transforms.Compose(clip_preprocess.transforms[-2:]))
    return item_tfms, batch_tfms

# %% ../../nbs/ml.clip.ipynb 6
class ClipClassificationHead(nn.Module):
    def __init__(self, clip_model):
        super().__init__()
        self.logit_scale = nn.Parameter(clip_model.logit_scale.detach().clone(), requires_grad=True) 
    
    def forward(self, image_features, text_features):
        image_features = image_features / image_features.norm(dim=-1, keepdim=True)
        text_features = text_features / text_features.norm(dim=-1, keepdim=True)
        logits = self.logit_scale.exp() * (image_features @ text_features.t())
        return logits

# %% ../../nbs/ml.clip.ipynb 7
class ClipZeroShotClassifier(nn.Module):
    def __init__(self, clip_model, class_descriptions):
        super().__init__()
        self.clip_model = clip_model
        self.head = ClipClassificationHead(clip_model)
        with torch.inference_mode():
            ctf = self.compute_text_features(class_descriptions)
        self.class_text_features = nn.Parameter(ctf, requires_grad=False)
    
    def forward(self, image):
        image_features = self.clip_model.encode_image(image)
        return self.head(image_features, self.class_text_features)

    def compute_text_features(self, texts):
        device = next(self.clip_model.parameters()).device
        text_tokens = clip.tokenize(texts)
        text_features = self.clip_model.encode_text(text_tokens.to(device)).float()
        return text_features

